/**
 *
 * @param prefix
 * @param connection
 * @param handler
 * @return A proxy that translates method calls on itself to requests sent on the {@link connection}.
 */
function createProxyAndHandleRequests(prefix, connection, handler) {
    handleRequests(connection, prefix, handler);
    return createProxy(connection, prefix);
}
function createExtensionProxy(connection, rpcId) {
    let handler = {
        get: (target, name) => {
            if (!target[name] && name[0] === '$') {
                const method = `${rpcId}/${name}`;
                target[name] = (...myArgs) => {
                    return connection.sendRequest(method, myArgs);
                };
            }
            return target[name];
        },
    };
    return new Proxy(Object.create(null), handler);
}
/**
 * Creates a Proxy that translates method calls (whose name begins with "$") on the returned object to messages
 * named `${prefix}/${name}` on the connectino.
 *
 * @param connection - The connection to send messages on when proxy methods are called.
 * @param prefix - The name prefix for connection methods.
 */
function createProxy(connection, prefix) {
    return new Proxy(Object.create(null), {
        get: (target, name) => {
            if (!target[name] && name[0] === '$') {
                const method = `${prefix}/${name}`;
                if (name.startsWith('$observe')) {
                    target[name] = (...args) => connection.observeRequest(method, args);
                }
                else {
                    target[name] = (...args) => connection.sendRequest(method, args, null);
                }
            }
            return target[name];
        },
    });
}
/**
 * Forwards all requests received on the connection to the corresponding method on the handler object. The
 * connection method `${prefix}/${name}` corresponds to `${name}` method on the handler object.
 *
 *
 * @param connection - connection
 * @param prefix  - connection prefix
 * @param handler - An instance of a class whose methods should be called when the connection receives
 *                  corresponding requests, or an object created with Object.create(null) (or otherwise with a null
 *                  prototype) whose properties contain functions to be called.
 */
function handleRequests(connection, prefix, handler) {
    // A class instance's methods are own, non-enumerable properties of its prototype.
    const proto = Object.getPrototypeOf(handler) || handler;
    for (const name of Object.getOwnPropertyNames(proto)) {
        const value = proto[name];
        if (name[0] === '$' && typeof value === 'function') {
            connection.onRequest(`${prefix}/${name}`, (args) => value.apply(handler, args));
        }
    }
}
/**
 * Forwards all requests received on the connection to the corresponding method on the server side.
 *
 * @param connection - connection
 * @param extContext - Remote Context, usually for connecting to Server side providers
 * @param identifier - Remote Actor Identifier.
 */
function handleServerProxyRequest(connection, remoteProtocol, identifier) {
    const aProxy = remoteProtocol.getProxy(identifier);
    connection.onRequest(`${identifier.sid}/*`, (args, signal, method) => {
        const methodName = method.split('/')[1];
        if (!methodName) {
            throw new Error('method is not defined');
        }
        return aProxy[methodName](...args, signal);
    });
}
/**
 * Forwards all requests received on the connection to the corresponding method on the server side.
 *
 * @param connection - connection
 * @param extContext - Remote Context, usually for connecting to Server side providers
 * @param identifier - Remote Actor Identifier.
 */
function handleServerProxyRequests(connection, remoteProtocol, identifiers) {
    for (let i in identifiers) {
        handleServerProxyRequest(connection, remoteProtocol, identifiers[i]);
    }
}export{createExtensionProxy,createProxy,createProxyAndHandleRequests,handleRequests,handleServerProxyRequest,handleServerProxyRequests};