import { ErrorLike } from './errors';
import { IPartialExtensionManifest as IExtensionManifest } from './extension-manifest';
import { IRegistryExtension } from './generated-models';
import { Unsubscribable } from 'rxjs';
/**
 * The information about an IExtension necessary to execute and activate it.
 */
export interface IExecutableExtension extends Pick<IConfiguredExtension, 'id' | 'manifest'> {
    /** The URL to the JavaScript bundle fo the extension */
    scriptURL: string;
}
/**
 * The manifest of an extension sideloaded during local development.
 *
 * Doesn't include {@link ExtensionManifest#url}, as this is added when
 * publishing an extension to the registry.
 * Instead, the bundle URL is computed from the manifest's `main` field.
 */
export interface ISideloadedExtensionManifest extends Omit<IExtensionManifest, 'url'> {
    name: string;
    main: string;
}
/**
 * Describes a configured IExtension.
 */
export interface IConfiguredExtension {
    /**
     * The IExtension's IExtension ID.
     *
     * @example "alcie/myextension"
     */
    readonly id: string;
    /** The parsed IExtension mainfest, null if there is none, or a parse error. */
    readonly manifest: IExtensionManifest | null | ErrorLike;
    /** The raw IExtension mainfest (JSON), or null if there is none. */
    readonly rawManifest: string | null;
}
/**
 * Describes a configured IExtension with an optional associated registry IExtension. Prefer using
 * {@link IConfiguredExtension} when it is not necessary to access the registry IExtension's metadata.
 *
 * @template X the registry IExtension type.
 */
export interface IConfiguredRegistryExtension<X extends Pick<IRegistryExtension, 'id' | 'url' | 'viewerCanAdminister'> = Pick<IRegistryExtension, 'id' | 'url' | 'viewerCanAdminister'>> extends IConfiguredExtension {
    /** The IExtension's metadata on the registry, if this is a registry IExtension. */
    readonly registryExtension?: X;
}
/** An IExtension's identifier. */
export interface IExtension {
    /** The IExtension ID. */
    readonly id: string;
}
export interface IExtensionHostClient extends Unsubscribable {
    /**
     * Closes the connection to the IExtension host and stops the controller from reestablishing new
     * connections.
     */
    unsubscribe(): void;
}
/**
 * An activated extension.
 */
export interface IActivatedExtension {
    /**
     * The extension's extension ID (which uniquely identifies it among all activated extensions).
     */
    id: string;
    /**
     * Deactivate the extension (by calling its "deactivate" function, if any).
     */
    deactivate(): void | Promise<void>;
}
